@@ -1,6 +1,6 @@
 /*
  *  Program: pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2005 David Barnes
+ *  Copyright (C) 1994-2007 David Barnes
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 1, or (at your option)
@@ -25,6 +25,7 @@ typedef enum {
     SEVEN_TAG_ROSTER_ARGUMENT = '7',
     GAMES_PER_FILE_ARGUMENT = '#',
     ALTERNATIVE_HELP_ARGUMENT = '?',
+    LONG_FORM_ARGUMENT = '-',
     APPEND_TO_OUTPUT_FILE_ARGUMENT = 'a',
     BOUNDS_ARGUMENT = 'b',
     CHECK_FILE_ARGUMENT = 'c',
@@ -48,6 +49,7 @@ typedef enum {
     ECO_OUTPUT_LEVEL_ARGUMENT = 'E',
     OUTPUT_FEN_STRING_ARGUMENT = 'F',
     APPEND_TO_LOG_FILE_ARGUMENT = 'L',
+    MATCH_CHECKMATE_ARGUMENT = 'M',
     DONT_KEEP_NAGS_ARGUMENT = 'N',
     DONT_MATCH_PERMUTATIONS_ARGUMENT = 'P',
     TAG_ROSTER_ARGUMENT = 'R',
@@ -57,8 +59,9 @@ typedef enum {
     DONT_KEEP_VARIATIONS_ARGUMENT = 'V',
     OUTPUT_FORMAT_ARGUMENT = 'W',
     USE_VIRTUAL_HASH_TABLE_ARGUMENT = 'Z',
-    NO_ARGUMENT_MATCH = '\0'	/* No argument match. */
+    NO_ARGUMENT_MATCH = '\0'        /* No argument match. */
 } ArgType;
 
 void read_args_file(const char *infile);
 void process_argument(char arg_letter,const char *associated_value);
+int process_long_form_argument(const char *argument, const char *associated_value);
