@@ -38,9 +38,9 @@ typedef enum {
 } Piece;
 /* Different classes of move determined by the lexical analyser. */
 typedef enum { PAWN_MOVE, PAWN_MOVE_WITH_PROMOTION, ENPASSANT_PAWN_MOVE,
-	       PIECE_MOVE, KINGSIDE_CASTLE, QUEENSIDE_CASTLE,
-	       UNKNOWN_MOVE
-	     } MoveClass;
+               PIECE_MOVE, KINGSIDE_CASTLE, QUEENSIDE_CASTLE,
+               UNKNOWN_MOVE
+             } MoveClass;
 /* Types for algebraic rank and column. */
 typedef char Rank;
 typedef char Col;
@@ -55,10 +55,10 @@ typedef char Col;
 
 /* Convert the given rank to the correct index into a board. */
 #define RankConvert(rank) ((FIRSTRANK <= (rank)) && ((rank) <= LASTRANK)?\
-					((rank)-RANKBASE+HEDGE):0)
+                                        ((rank)-RANKBASE+HEDGE):0)
 /* Convert the given column to the correct index into a board. */
 #define ColConvert(col) ((FIRSTCOL <= (col)) && ((col) <= LASTCOL)?\
-					((col)-COLBASE+HEDGE):0)
+                                        ((col)-COLBASE+HEDGE):0)
 
 /* Convert a board index back to Rank or Col form. */
 #define ToRank(r) ((r)+RANKBASE-HEDGE)
@@ -95,6 +95,8 @@ typedef struct {
     Rank ep_rank;
     Col ep_col;
     HashCode hash_value;
+    /* The half-move clock since the last pawn move or capture. */
+    unsigned halfmove_clock;
 } Board;
 
 /* Define a type that can be used to create a list of possible source
