@@ -20,7 +20,7 @@
  *
  */
 
-	/* Define a type to hold hash values of interest. */
+        /* Define a type to hold hash values of interest. */
 typedef struct EcoLog {
     HashCode required_hash_value;
     /* cumulative_hash_value is used to disambiguate clashing
@@ -39,7 +39,7 @@ typedef struct EcoLog {
 } EcoLog;
 
 EcoLog *eco_matches(HashCode current_hash_value, HashCode cumulative_hash_value,
-		    int half_moves_played);
+                    unsigned half_moves_played);
 Boolean add_ECO(Game game_details);
 FILE *open_eco_output_file(EcoDivision ECO_level,const char *eco);
 void initEcoTable(void);
