@@ -24,7 +24,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
-#ifdef __BORLANDC__
+#if defined(__BORLANDC__) || defined(_MSC_VER)
 #include <io.h>
 #ifndef R_OK
 #define R_OK 0
@@ -58,7 +58,7 @@ static void save_k_castle(void);
 static void terminate_input(void);
 static char *next_input_line(FILE *fp);
 static Boolean extract_yytext(const unsigned char *symbol_start,
-			      const unsigned char *linep);
+                              const unsigned char *linep);
 static Boolean open_input(const char *infile);
 static Boolean open_input_file(int file_number);
 
@@ -174,7 +174,7 @@ make_new_tag(const char *tag)
 {   int tag_index = tag_list_length;
     tag_list_length++;
     TagList = (const char **) ReallocOrDie((void *)TagList,
-		    tag_list_length*sizeof(*TagList));
+                    tag_list_length*sizeof(*TagList));
     TagList[tag_index] = StringCopy(tag);
     /* Ensure that the game header's tags array can accommodate
      * the new tag.
@@ -507,8 +507,8 @@ identify_tag(const char *tag_string)
 
     for(tag_index = 0; tag_index < tag_list_length; tag_index++){
         if(strcmp(tag_string,TagList[tag_index]) == 0){
-	    return tag_index;
-	}
+            return tag_index;
+        }
     }
     /* Not found. */
     return -1;
@@ -529,9 +529,11 @@ gather_tag(char *line, unsigned char *linep)
             line = next_input_line(yyin);
             linep = (unsigned char *) line;
         }
-        while(ChTab[(unsigned)*linep] == WHITESPACE){
-            linep++;
-        }
+	if(line != NULL) {
+	    while(ChTab[(unsigned)*linep] == WHITESPACE){
+		linep++;
+	    }
+	}
     }
     while((line != NULL) && (ChTab[(unsigned)*linep] == '\0'));
 
@@ -552,19 +554,19 @@ gather_tag(char *line, unsigned char *linep)
             strncpy((char *)tag_string,(const char *)(linep-len),len);
             tag_string[len] = '\0';
             tag_item = identify_tag(tag_string);
-	    if(tag_item < 0){
-		tag_item = make_new_tag(tag_string);
-	    }
+            if(tag_item < 0){
+                tag_item = make_new_tag(tag_string);
+            }
             if(tag_item >= 0 && tag_item < tag_list_length){
                 yylval.tag_index = tag_item;
                 resulting_line.token = TAG;
-		(void) free((void *)tag_string);
+                (void) free((void *)tag_string);
             }
             else{
-		fprintf(GlobalState.logfile,
-		    "Internal error: invalid tag index %d in gather_tag.\n",
-		    tag_item);
-		exit(1);
+                fprintf(GlobalState.logfile,
+                    "Internal error: invalid tag index %d in gather_tag.\n",
+                    tag_item);
+                exit(1);
             }
         }
         else{
@@ -583,7 +585,7 @@ static Boolean
 extract_yytext(const unsigned char *symbol_start,const unsigned char *linep)
 {   /* Whether the string fitted. */
     Boolean Ok = TRUE;
-    unsigned len = linep-symbol_start;
+    long len = linep-symbol_start;
 
     if(len < MAX_YYTEXT){
         strncpy((char *) yytext,(const char *) symbol_start,len);
@@ -776,8 +778,8 @@ get_next_symbol(void)
                         if(!skipping_game){
                             print_error_context(GlobalState.logfile);
                             fprintf(GlobalState.logfile,
-				    "Unknown character %c (Hex: %x).\n",
-				    next_char,next_char);
+                                    "Unknown character %c (Hex: %x).\n",
+                                    next_char,next_char);
                         }
                         /* Skip any sequence of them. */
                         while(ChTab[(unsigned)*linep] == ERROR_TOKEN)
@@ -789,7 +791,7 @@ get_next_symbol(void)
                      * Remember that 1 can start 1-0 and 1/2.
                      */
                     resulting_line = gather_possible_numeric(
-					line,linep,next_char);
+                                        line,linep,next_char);
                     /* Pick up where we are now. */
                     line = resulting_line.line;
                     linep = resulting_line.linep;
@@ -825,9 +827,9 @@ get_next_symbol(void)
                 case ERROR_TOKEN:
                     if(!skipping_game){
                         print_error_context(GlobalState.logfile);
-			fprintf(GlobalState.logfile,
-				"Unknown character %c (Hex: %x).\n",
-				next_char,next_char);
+                        fprintf(GlobalState.logfile,
+                                "Unknown character %c (Hex: %x).\n",
+                                next_char,next_char);
                     }
                     /* Skip any sequence of them. */
                     while(ChTab[(unsigned)*linep] == ERROR_TOKEN)
@@ -847,7 +849,7 @@ get_next_symbol(void)
                         print_error_context(GlobalState.logfile);
                         fprintf(GlobalState.logfile,
                             "Internal error: Missing case for %d on char %x.\n",
-			    token,next_char);
+                            token,next_char);
                     }
                     token = NO_TOKEN;
                     break;
@@ -861,9 +863,9 @@ TokenType
 next_token(void)
 {   TokenType token = get_next_symbol();
 
-	/* Don't call yywrap if parsing the ECO file. */
+        /* Don't call yywrap if parsing the ECO file. */
     while((token == EOF_TOKEN) && !GlobalState.parsing_ECO_file &&
-			!yywrap()){
+                        !yywrap()){
         token = get_next_symbol();
     }
     return token;
@@ -877,12 +879,12 @@ skip_token(TokenType token)
 {
     switch(token){
         case TERMINATING_RESULT:
-	case TAG:
-	case MOVE:
-	case EOF_TOKEN:
-	    return FALSE;
-	default:
-	    return TRUE;
+        case TAG:
+        case MOVE:
+        case EOF_TOKEN:
+            return FALSE;
+        default:
+            return TRUE;
     }
 }
 
@@ -946,7 +948,7 @@ restart_lex_for_new_game(void)
 
         /* Make it possible to read multiple input files.
          * These are held in list_of_files. The list
-	 * is built up from the program's arguments.
+         * is built up from the program's arguments.
          */
 static int current_file_num = 0;
     /* Keep track of the list of PGN files.  These will either be the
@@ -954,9 +956,9 @@ static int current_file_num = 0;
      * those read from -c and -f arguments.
      */
 static FILE_LIST list_of_files = {
-	(const char **) NULL,
-	(SourceFileType *) NULL,
-	0, 0
+        (const char **) NULL,
+        (SourceFileType *) NULL,
+        0, 0
 };
 
     /* Return the index number of the current input file in list_of_files. */
@@ -967,105 +969,105 @@ current_file_number(void)
 }
 
 
-	/* Read a list of lines from fp. These are the names of files
-	 * to be added to the existing list_of_files.
-	 * list_of_files.list must have a (char *)NULL on the end.
-	 */
+        /* Read a list of lines from fp. These are the names of files
+         * to be added to the existing list_of_files.
+         * list_of_files.list must have a (char *)NULL on the end.
+         */
 void
 add_filename_list_from_file(FILE *fp,SourceFileType file_type)
 {
     if((list_of_files.files == NULL) || (list_of_files.max_files == 0)){
-	 /* Allocate an initial number of pointers for the lines.
-	  * This must always include an extra one for terminating NULL.
-	  */
-	 list_of_files.files = (const char **) MallocOrDie((INIT_LIST_SPACE+1)*
-						sizeof(const char *));
-	 list_of_files.file_type = (SourceFileType *) MallocOrDie((INIT_LIST_SPACE+1)*
-						sizeof(SourceFileType));
-	 list_of_files.max_files = INIT_LIST_SPACE;
-	 list_of_files.num_files = 0;
+         /* Allocate an initial number of pointers for the lines.
+          * This must always include an extra one for terminating NULL.
+          */
+         list_of_files.files = (const char **) MallocOrDie((INIT_LIST_SPACE+1)*
+                                                sizeof(const char *));
+         list_of_files.file_type = (SourceFileType *) MallocOrDie((INIT_LIST_SPACE+1)*
+                                                sizeof(SourceFileType));
+         list_of_files.max_files = INIT_LIST_SPACE;
+         list_of_files.num_files = 0;
     }
     if(list_of_files.files != NULL){
-	/* Find the first line. */
-	char *line = read_line(fp);
+        /* Find the first line. */
+        char *line = read_line(fp);
 
-	while(line != NULL){
-	     if(non_blank_line(line)){
-		 add_filename_to_source_list(line,file_type);
-	     }
-	     else{
-	         (void) free((void *)line);
-	     }
-	     line = read_line(fp);
-	}
+        while(line != NULL){
+             if(non_blank_line(line)){
+                 add_filename_to_source_list(line,file_type);
+             }
+             else{
+                 (void) free((void *)line);
+             }
+             line = read_line(fp);
+        }
      }
 }
 
 void
 add_filename_to_source_list(const char *filename,SourceFileType file_type)
 {    /* Where to put it. */
-     int location = list_of_files.num_files;
+     unsigned location = list_of_files.num_files;
 
      if(access(filename,R_OK) != 0){
-	 fprintf(GlobalState.logfile,"Unable to find %s\n",filename);
-	 exit(1);
+         fprintf(GlobalState.logfile,"Unable to find %s\n",filename);
+         exit(1);
      }
      else{
-	 /* Ok. */
+         /* Ok. */
      }
      /* See if there is room. */
      if(list_of_files.num_files == list_of_files.max_files){
-	 /* There isn't, so increase the amount of available space,
-	  * ensuring that there is always an extra slot for the terminating
-	  * NULL.
-	  */
-	if((list_of_files.files == NULL) || (list_of_files.max_files == 0)){
-	     /* Allocate an initial number of pointers for the lines.
-	      * This must always include an extra one for terminating NULL.
-	      */
-	     list_of_files.files = (const char **) MallocOrDie((INIT_LIST_SPACE+1)*
-						    sizeof(const char *));
-	     list_of_files.file_type = (SourceFileType *)
-				MallocOrDie((INIT_LIST_SPACE+1)*
-						    sizeof(SourceFileType));
-	     list_of_files.max_files = INIT_LIST_SPACE;
-	     list_of_files.num_files = 0;
-	}
-	else{
-	     list_of_files.files = (const char **)realloc((void *)list_of_files.files,
-		    (list_of_files.max_files+MORE_LIST_SPACE+1)*
-						    sizeof(const char *));
-	     list_of_files.file_type = (SourceFileType *)
-		    realloc((void *)list_of_files.file_type,
-		    (list_of_files.max_files+MORE_LIST_SPACE+1)*
-						    sizeof(SourceFileType));
-	     list_of_files.max_files += MORE_LIST_SPACE;
-	    if((list_of_files.files == NULL) && (list_of_files.file_type == NULL)){
-		perror("");
-		abort();
-	    }
-	}
+         /* There isn't, so increase the amount of available space,
+          * ensuring that there is always an extra slot for the terminating
+          * NULL.
+          */
+        if((list_of_files.files == NULL) || (list_of_files.max_files == 0)){
+             /* Allocate an initial number of pointers for the lines.
+              * This must always include an extra one for terminating NULL.
+              */
+             list_of_files.files = (const char **) MallocOrDie((INIT_LIST_SPACE+1)*
+                                                    sizeof(const char *));
+             list_of_files.file_type = (SourceFileType *)
+                                MallocOrDie((INIT_LIST_SPACE+1)*
+                                                    sizeof(SourceFileType));
+             list_of_files.max_files = INIT_LIST_SPACE;
+             list_of_files.num_files = 0;
+        }
+        else{
+             list_of_files.files = (const char **)realloc((void *)list_of_files.files,
+                    (list_of_files.max_files+MORE_LIST_SPACE+1)*
+                                                    sizeof(const char *));
+             list_of_files.file_type = (SourceFileType *)
+                    realloc((void *)list_of_files.file_type,
+                    (list_of_files.max_files+MORE_LIST_SPACE+1)*
+                                                    sizeof(SourceFileType));
+             list_of_files.max_files += MORE_LIST_SPACE;
+            if((list_of_files.files == NULL) && (list_of_files.file_type == NULL)){
+                perror("");
+                abort();
+            }
+        }
      }
      /* We know that there is space. Ensure that CHECKFILEs are all
       * stored before NORMALFILEs.
       */
      if(file_type == CHECKFILE){
 
-	 for(location = 0; (location < list_of_files.num_files) &&
-			(list_of_files.file_type[location] == CHECKFILE); location++){
-	    /* Do nothing. */
-	 }
-	 if(location < list_of_files.num_files){
-	    /* Put the new one here.
-	     * Move the rest down.
-	     */
-	    int j;
+         for(location = 0; (location < list_of_files.num_files) &&
+                        (list_of_files.file_type[location] == CHECKFILE); location++){
+            /* Do nothing. */
+         }
+         if(location < list_of_files.num_files){
+            /* Put the new one here.
+             * Move the rest down.
+             */
+            unsigned j;
 
-	    for(j = list_of_files.num_files; j > location; j--){
-		list_of_files.files[j] = list_of_files.files[j-1];
-		list_of_files.file_type[j] = list_of_files.file_type[j-1];
-	    }
-	}
+            for(j = list_of_files.num_files; j > location; j--){
+                list_of_files.files[j] = list_of_files.files[j-1];
+                list_of_files.file_type[j] = list_of_files.file_type[j-1];
+            }
+        }
     }
     list_of_files.files[location] = filename;
     list_of_files.file_type[location] = file_type;
@@ -1121,9 +1123,9 @@ open_first_file(void)
         /* Use standard input. */
         yyin = stdin;
         GlobalState.current_input_file = "stdin";
-	/* @@@ Should this be set?
-	GlobalState.current_file_type = NORMALFILE;
-	 */
+        /* @@@ Should this be set?
+        GlobalState.current_file_type = NORMALFILE;
+         */
         if(GlobalState.verbose){
             fprintf(GlobalState.logfile,"Processing %s\n",
                                 GlobalState.current_input_file);
@@ -1168,7 +1170,7 @@ print_error_context(FILE *fp)
         /* Save the given str accessible to YACC. */
 static void
 save_string(const char *str)
-{   const int len = strlen(str);
+{   const size_t len = strlen(str);
     char *token;
 
     token = MallocOrDie(len+1);
@@ -1264,12 +1266,12 @@ read_tag_file(const char *TagFile)
         
         while(keep_reading){
             char *line = next_input_line(yyin);
-	    if(line != NULL){
-	        keep_reading = process_tag_line(TagFile,line);
+            if(line != NULL){
+                keep_reading = process_tag_line(TagFile,line);
+            }
+            else{
+                keep_reading = FALSE;
             }
-	    else{
-	        keep_reading = FALSE;
-	    }
         }
         (void) fclose(yyin);
         /* Call yywrap in order to set up for the next (first) input file. */
@@ -1277,174 +1279,174 @@ read_tag_file(const char *TagFile)
     }
     else{
         fprintf(GlobalState.logfile,
-		"Unable to open %s for reading.\n",TagFile);
+                "Unable to open %s for reading.\n",TagFile);
         exit(1);
     }
 }
 
-	/* Read the contents of a file that lists the
-	 * required output ordering for tags.
-	 */
+        /* Read the contents of a file that lists the
+         * required output ordering for tags.
+         */
 void
 read_tag_roster_file(const char *RosterFile)
 {   Boolean keep_reading = TRUE;
     yyin = MustOpen(RosterFile,"r");
     
     while(keep_reading){
-	char *line = next_input_line(yyin);
-	if(line != NULL){
-	    keep_reading = process_roster_line(line);
-	}
-	else{
-	    keep_reading = FALSE;
-	}
+        char *line = next_input_line(yyin);
+        if(line != NULL){
+            keep_reading = process_roster_line(line);
+        }
+        else{
+            keep_reading = FALSE;
+        }
     }
     (void) fclose(yyin);
     /* Call yywrap in order to set up for the next (first) input file. */
     (void) yywrap();
 }
 
-	/* Extract a tag/value pair from the given line.
-	 * Return TRUE if this was successful.
-	 */
+        /* Extract a tag/value pair from the given line.
+         * Return TRUE if this was successful.
+         */
 Boolean
 process_tag_line(const char *TagFile,char *line)
 {
     Boolean keep_reading = TRUE;
     if(non_blank_line(line)){
-	unsigned char *linep = (unsigned char *) line;
-	/* We should find a tag. */
-	LinePair resulting_line = gather_tag(line,linep);
-	TokenType tag_token;
+        unsigned char *linep = (unsigned char *) line;
+        /* We should find a tag. */
+        LinePair resulting_line = gather_tag(line,linep);
+        TokenType tag_token;
 
-	/* Pick up where we are now. */
-	line = resulting_line.line;
-	linep = resulting_line.linep;
-	tag_token = resulting_line.token;
-	if(tag_token != NO_TOKEN){
-	    /* Pick up which tag it was. */
-	    int tag_index = yylval.tag_index;
-	    /* Allow for an optional operator. */
-	    TagOperator operator = NONE;
+        /* Pick up where we are now. */
+        line = resulting_line.line;
+        linep = resulting_line.linep;
+        tag_token = resulting_line.token;
+        if(tag_token != NO_TOKEN){
+            /* Pick up which tag it was. */
+            int tag_index = yylval.tag_index;
+            /* Allow for an optional operator. */
+            TagOperator operator = NONE;
 
-	    /* Skip whitespace up to a double quote. */
-	    while(ChTab[(unsigned)*linep] == WHITESPACE){
-		linep++;
-	    }
-	    /* Allow for an optional operator. */
-	    if(ChTab[(unsigned)*linep] == OPERATOR){
-		switch(*linep){
-		  case '<':
-		    linep++;
-		    if(*linep == '='){
-			linep++;
-			operator = LESS_THAN_OR_EQUAL_TO;
-		    }
-		    else if(*linep == '>'){
-			linep++;
-			operator = NOT_EQUAL_TO;
-		    }
-		    else{
-			operator = LESS_THAN;;
-		    }
-		    break;
-		  case '>':
-		    linep++;
-		    if(*linep == '='){
-			linep++;
-			operator = GREATER_THAN_OR_EQUAL_TO;
-		    }
-		    else{
-			operator = GREATER_THAN;
-		    }
-		    break;
-		  case '=':
-		    linep++;
-		    operator = EQUAL_TO;
-		    break;
-		  default:
-		    fprintf(GlobalState.logfile,
-			"Internal error: unknown operator in %s\n",line);
-		    linep++;
-		    break;
-		}
-		/* Skip whitespace up to a double quote. */
-		while(ChTab[(unsigned)*linep] == WHITESPACE){
-		    linep++;
-		}
-	    }
+            /* Skip whitespace up to a double quote. */
+            while(ChTab[(unsigned)*linep] == WHITESPACE){
+                linep++;
+            }
+            /* Allow for an optional operator. */
+            if(ChTab[(unsigned)*linep] == OPERATOR){
+                switch(*linep){
+                  case '<':
+                    linep++;
+                    if(*linep == '='){
+                        linep++;
+                        operator = LESS_THAN_OR_EQUAL_TO;
+                    }
+                    else if(*linep == '>'){
+                        linep++;
+                        operator = NOT_EQUAL_TO;
+                    }
+                    else{
+                        operator = LESS_THAN;;
+                    }
+                    break;
+                  case '>':
+                    linep++;
+                    if(*linep == '='){
+                        linep++;
+                        operator = GREATER_THAN_OR_EQUAL_TO;
+                    }
+                    else{
+                        operator = GREATER_THAN;
+                    }
+                    break;
+                  case '=':
+                    linep++;
+                    operator = EQUAL_TO;
+                    break;
+                  default:
+                    fprintf(GlobalState.logfile,
+                        "Internal error: unknown operator in %s\n",line);
+                    linep++;
+                    break;
+                }
+                /* Skip whitespace up to a double quote. */
+                while(ChTab[(unsigned)*linep] == WHITESPACE){
+                    linep++;
+                }
+            }
 
-	    if(ChTab[(unsigned)*linep] == DOUBLE_QUOTE){
-		/* A string, as expected. */
-		linep++;
-		resulting_line = gather_string(line,linep);
-		line = resulting_line.line;
-		linep = resulting_line.linep;
-		if(tag_token == TAG){
-		    /* Treat FEN tags as a special case.
-		     * Use the position they represent to indicate
-		     * a positional match.
-		     */
-		    if(tag_index == FEN_TAG){
-			add_fen_positional_match(yylval.token_string);
-			(void) free((void *)yylval.token_string);
-		    }
-		    else{
-			add_tag_to_list(tag_index,yylval.token_string,operator);
-			(void) free((void *)yylval.token_string);
-		    }
-		}
-		else{
-		    if(!skipping_game){
-			    fprintf(GlobalState.logfile,
-				"File %s: unrecognised tag name %s\n",
-					TagFile,line);
-		    }
-		    (void) free((void *)yylval.token_string);
-		}
-	    }
-	    else{
-		if(!skipping_game){
-		    fprintf(GlobalState.logfile,
-			    "File %s: missing quoted tag string in %s at %s\n",
-			    TagFile,line,linep);
-	        }
-	    }
-	}
-	else{
-	    /* Terminate the reading, as we have run out of tags. */
-	    keep_reading = FALSE;
-	}
+            if(ChTab[(unsigned)*linep] == DOUBLE_QUOTE){
+                /* A string, as expected. */
+                linep++;
+                resulting_line = gather_string(line,linep);
+                line = resulting_line.line;
+                linep = resulting_line.linep;
+                if(tag_token == TAG){
+                    /* Treat FEN tags as a special case.
+                     * Use the position they represent to indicate
+                     * a positional match.
+                     */
+                    if(tag_index == FEN_TAG){
+                        add_fen_positional_match(yylval.token_string);
+                        (void) free((void *)yylval.token_string);
+                    }
+                    else{
+                        add_tag_to_list(tag_index,yylval.token_string,operator);
+                        (void) free((void *)yylval.token_string);
+                    }
+                }
+                else{
+                    if(!skipping_game){
+                            fprintf(GlobalState.logfile,
+                                "File %s: unrecognised tag name %s\n",
+                                        TagFile,line);
+                    }
+                    (void) free((void *)yylval.token_string);
+                }
+            }
+            else{
+                if(!skipping_game){
+                    fprintf(GlobalState.logfile,
+                            "File %s: missing quoted tag string in %s at %s\n",
+                            TagFile,line,linep);
+                }
+            }
+        }
+        else{
+            /* Terminate the reading, as we have run out of tags. */
+            keep_reading = FALSE;
+        }
     }
     return keep_reading;
 }
 
-	/* Extract a tag name from the given line.
-	 * Return TRUE if this was successful.
-	 */
+        /* Extract a tag name from the given line.
+         * Return TRUE if this was successful.
+         */
 Boolean
 process_roster_line(char *line)
 {
     Boolean keep_reading = TRUE;
     if(non_blank_line(line)){
-	unsigned char *linep = (unsigned char *) line;
-	/* We should find a tag. */
-	LinePair resulting_line = gather_tag(line,linep);
-	TokenType tag_token;
+        unsigned char *linep = (unsigned char *) line;
+        /* We should find a tag. */
+        LinePair resulting_line = gather_tag(line,linep);
+        TokenType tag_token;
 
-	/* Pick up where we are now. */
-	line = resulting_line.line;
-	linep = resulting_line.linep;
-	tag_token = resulting_line.token;
-	if(tag_token != NO_TOKEN){
-	    /* Pick up which tag it was. */
-	    int tag_index = yylval.tag_index;
-	    add_to_output_tag_order((TagName) tag_index);
-	}
-	else{
-	    /* Terminate the reading, as we have run out of tags. */
-	    keep_reading = FALSE;
-	}
+        /* Pick up where we are now. */
+        line = resulting_line.line;
+        linep = resulting_line.linep;
+        tag_token = resulting_line.token;
+        if(tag_token != NO_TOKEN){
+            /* Pick up which tag it was. */
+            int tag_index = yylval.tag_index;
+            add_to_output_tag_order((TagName) tag_index);
+        }
+        else{
+            /* Terminate the reading, as we have run out of tags. */
+            keep_reading = FALSE;
+        }
     }
     return keep_reading;
 }
