@@ -56,8 +56,6 @@ typedef struct {
 void yyerror(const char *s);
 void save_assessment(const char *assess);
 void restart_lex_for_new_game(void);
-void *MallocOrDie(unsigned nbytes);
-void *ReallocOrDie(void *space,unsigned nbytes);
 void free_move_list(Move *move_list);
 void read_tag_file(const char *TagFile);
 void read_tag_roster_file(const char *RosterFile);
@@ -68,7 +66,6 @@ void init_lex_tables(void);
 TokenType next_token(void);
 TokenType skip_to_next_game(TokenType token);
 const char *tag_header_string(TagName tag);
-int stricmp(const char *,const char *);
 Boolean open_first_file(void);
 const char *input_file_name(int file_number);
 int current_file_number(void);
