@@ -1,6 +1,6 @@
 /*
  *  Program: pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2005 David Barnes
+ *  Copyright (C) 1994-2007 David Barnes
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 1, or (at your option)
@@ -64,7 +64,7 @@ StateInfo GlobalState = {
     TRUE,                 /* keep_NAGs (-N) */
     TRUE,                 /* keep_comments (-C) */
     TRUE,                 /* keep_variations (-V) */
-    FALSE,                /* seven_tag_roster (-7) */
+    ALL_TAGS,             /* tag_output_form (-7, --notags) */
     TRUE,                 /* match_permutations (-v) */
     FALSE,                /* positional_variations (-x) */
     FALSE,                /* use_soundex (-S) */
@@ -78,6 +78,11 @@ StateInfo GlobalState = {
     MAX_LINE_LENGTH,      /* max_line_length (-w) */
     FALSE,                /* use_virtual_hash_table (-Z) */
     FALSE,                /* check_move_bounds (-b) */
+    FALSE,                /* check_for_checkmate (-M) */
+    FALSE,                /* check_for_stalemate (--stalemate) */
+    TRUE,                 /* keep_move_numbers (--nomovenumbers) */
+    TRUE,                 /* keep_results (--noresults) */
+    FALSE,                /* output_evaluation (--evaluation) */
     0,                    /* depth_of_positional_search */
     0,                    /* num_games_processed */
     0,                    /* num_games_matched */
@@ -85,7 +90,8 @@ StateInfo GlobalState = {
     1,                    /* next_file_number */
     0,                    /* lower_move_bound */
     10000,                /* upper_move_bound */
-    FALSE,		  /* output_FEN_string */
+    0,                    /* output_ply_limit (--plylimit) */
+    FALSE,                /* output_FEN_string */
     (char *)NULL,         /* current_input_file */
     NORMALFILE,           /* current_file_type */
     DEFAULT_ECO_FILE,     /* eco_file (-e) */
@@ -120,178 +126,195 @@ main(int argc, char *argv[])
     /* Initialise the lexical analyser's tables. */
     init_lex_tables();
     /* Allow for some arguments. */
-    for(argnum = 1; argnum < argc; argnum++){
+    for(argnum = 1; argnum < argc; ){
         const char *argument = argv[argnum];
         if(argument[0] == '-'){
             switch(argument[1]){
+                /* Arguments with no additional component. */
                 case SEVEN_TAG_ROSTER_ARGUMENT:
-		    process_argument(argument[1],"");
-                    break;
-                case TAG_ROSTER_ARGUMENT:
-		    process_argument(argument[1],&argument[2]);
-                    break;
-                case FILE_OF_ARGUMENTS_ARGUMENT:
-                   {  /* We expect a file name containing arguments. */
-                      const char *filename = &(argument[2]);
-		      process_argument(argument[1],filename);
-                    }
-                    break;
-                case APPEND_TO_OUTPUT_FILE_ARGUMENT:
-                    { /* We expect an output file to which to append. */
-                      const char *filename = &(argument[2]);
-		      process_argument(argument[1],filename);
-		    }
-                    break;
-                case BOUNDS_ARGUMENT:
-                    { const char *bound = &(argument[2]);
-		      process_argument(argument[1],bound);
-                  }
-                  break;
-                case CHECK_FILE_ARGUMENT:
-                    { /* We expect a list of files as the game sources.
-                       * These files are CHECKFILEs, to act as checks
-                       * for duplicates in any NORMALFILEs.
-                       */
-                      const char *filename = &(argument[2]);
-		      process_argument(argument[1],filename);
-                    }
-                    break;
                 case DONT_KEEP_COMMENTS_ARGUMENT:
-		    process_argument(argument[1],"");
-                    break;
-                case DUPLICATES_FILE_ARGUMENT:
-                    { /* We expect an output file for duplicate games. */
-                      const char *filename = &(argument[2]);
-		      process_argument(argument[1],filename);
-                    }
-                    break;
                 case DONT_KEEP_DUPLICATES_ARGUMENT:
-		    process_argument(argument[1],"");
-                    break;
-                case USE_ECO_FILE_ARGUMENT:
-                    { /* A filename of ECO lines is optional. */
-                      const char *filename = &(argument[2]);
-		      process_argument(argument[1],filename);
-                    }
-                    break;
-                case ECO_OUTPUT_LEVEL_ARGUMENT:
-                    /* Set an ECO output level. */
-		    process_argument(argument[1],&argument[2]);
-                    break;
-                case FILE_OF_FILES_ARGUMENT:
-                    { /* We expect a list of files as the game sources. */
-                      const char *filename = &(argument[2]);
-		      process_argument(argument[1],filename);
-                    }
-                    break;
-		case OUTPUT_FEN_STRING_ARGUMENT:
-		    process_argument(argument[1],"");
-		    break;
-                case HELP_ARGUMENT:
-		    process_argument(argument[1],&argument[2]);
-                    break;
-                case ALTERNATIVE_HELP_ARGUMENT:
-		    process_argument(HELP_ARGUMENT,"");
-                    break;
-                case WRITE_TO_LOG_FILE_ARGUMENT:
-		    /* Create new log file. */
-                case APPEND_TO_LOG_FILE_ARGUMENT:
-		    /* Append to log file. */
-		    process_argument(argument[1],&argument[2]);
-                    break;
-                case NON_MATCHING_GAMES_ARGUMENT:
-                    { /* We expect an output file for unmatched games. */
-                      const char *filename = &(argument[2]);
-		      process_argument(argument[1],filename);
-                    }
-                    break;
-                case DONT_KEEP_NAGS_ARGUMENT:
-		    process_argument(argument[1],"");
-                    break;
-                case WRITE_TO_OUTPUT_FILE_ARGUMENT:
-                    { /* We expect an output file to write to. */
-                      const char *filename = &(argument[2]);
-		      process_argument(argument[1],filename);
-                    }
-                    break;
-                case DONT_MATCH_PERMUTATIONS_ARGUMENT:
-		    process_argument(argument[1],"");
-                    break;
-                case CHECK_ONLY_ARGUMENT:
-		    process_argument(argument[1],"");
-                    break;
-                case KEEP_SILENT_ARGUMENT:
-		    process_argument(argument[1],"");
-                    break;
-                case USE_SOUNDEX_ARGUMENT:
-                    /* Use soundex matches for player tags. */
-		    process_argument(argument[1],"");
-                    break;
-                case TAG_EXTRACTION_ARGUMENT:
-                    /* A single tag extraction criterion. */
-		    process_argument(argument[1],&argument[2]);
-                    break;
-                case SUPPRESS_ORIGINALS_ARGUMENT:
-		    process_argument(argument[1],"");
-                    break;
                 case DONT_KEEP_VARIATIONS_ARGUMENT:
-		    process_argument(argument[1],"");
+                case DONT_KEEP_NAGS_ARGUMENT:
+                case DONT_MATCH_PERMUTATIONS_ARGUMENT:
+                case CHECK_ONLY_ARGUMENT:
+                case KEEP_SILENT_ARGUMENT:
+                case USE_SOUNDEX_ARGUMENT:
+                case MATCH_CHECKMATE_ARGUMENT:
+                case SUPPRESS_ORIGINALS_ARGUMENT:
+                case OUTPUT_FEN_STRING_ARGUMENT:
+                case USE_VIRTUAL_HASH_TABLE_ARGUMENT:
+                    process_argument(argument[1], "");
+                    argnum++;
                     break;
-                case LINE_WIDTH_ARGUMENT:
-		    process_argument(argument[1],&argument[2]);
+
+                /* Argument rewritten as a different one. */
+                case ALTERNATIVE_HELP_ARGUMENT:
+                    process_argument(HELP_ARGUMENT, "");
+                    argnum++;
                     break;
+
+                /* Arguments where an additional component is required.
+                 * It must be adjacent to the argument and not separated from it.
+                 */
+                case TAG_EXTRACTION_ARGUMENT:
+                    process_argument(argument[1], &(argument[2]));
+                    argnum++;
+                    break;
+
+                /* Arguments where an additional component is optional.
+                 * If it is present, it must be adjacent to the argument
+                 * letter and not separated from it.
+                 */
+                case USE_ECO_FILE_ARGUMENT:
                 case OUTPUT_FORMAT_ARGUMENT:
-		    process_argument(argument[1],&argument[2]);
+                case HELP_ARGUMENT:
+                    process_argument(argument[1], &(argument[2]));
+                    argnum++;
                     break;
+
+                /* Long form arguments. */
+                case LONG_FORM_ARGUMENT:
+                    {  
+                      /* How many args (1 or 2) are processed. */
+                      int args_processed;
+                      /* This argument might need the following argument
+                       * as an associated value.
+                       */
+                      const char *possible_associated_value = "";
+                      if(argnum + 1 < argc) {
+                          possible_associated_value = argv[argnum+1];
+                      }
+                      /* Find out how many arguments were consumed
+                       * (1 or 2).
+                       */
+                      args_processed =
+                          process_long_form_argument(&argument[2],
+                                                     possible_associated_value);
+                      argnum += args_processed;
+                    }
+                    break;
+
+                /* Arguments with a required filename component. */
+                case FILE_OF_ARGUMENTS_ARGUMENT:
+                case APPEND_TO_OUTPUT_FILE_ARGUMENT:
+                case CHECK_FILE_ARGUMENT:
+                case DUPLICATES_FILE_ARGUMENT:
+                case FILE_OF_FILES_ARGUMENT:
+                case WRITE_TO_LOG_FILE_ARGUMENT:
+                case APPEND_TO_LOG_FILE_ARGUMENT:
+                case NON_MATCHING_GAMES_ARGUMENT:
+                case WRITE_TO_OUTPUT_FILE_ARGUMENT:
+                case TAG_ROSTER_ARGUMENT:
+                    {   /* We require an associated file argument. */
+                        const char argument_letter = argument[1];
+                        const char *filename = &(argument[2]);
+                        if(*filename == '\0'){
+                             /* Try to pick it up from the next argument. */
+                             argnum++;
+                             if(argnum < argc){
+                                 filename = argv[argnum];
+                                 argnum++;
+                             }
+                            /* Make sure the associated_value does not look
+                             * like the next argument.
+                             */
+                            if((*filename == '\0') || (*filename == '-')){
+                                fprintf(GlobalState.logfile,
+                                        "Usage: -%c filename\n",
+                                        argument_letter);
+                                exit(1);
+                            }
+                        }
+                        else {
+                            argnum++;
+                        }
+                        process_argument(argument[1], filename);
+                    }
+                    break;
+
+                /* Arguments with a required followng value. */
+                case BOUNDS_ARGUMENT:
+                case ECO_OUTPUT_LEVEL_ARGUMENT:
+                case LINE_WIDTH_ARGUMENT:
+                case GAMES_PER_FILE_ARGUMENT:
+                    {   /* We require an associated file argument. */
+                        const char argument_letter = argument[1];
+                        const char *associated_value = &(argument[2]);
+                        if(*associated_value == '\0'){
+                             /* Try to pick it up from the next argument. */
+                             argnum++;
+                             if(argnum < argc){
+                                 associated_value = argv[argnum];
+                                 argnum++;
+                             }
+                            /* Make sure the associated_value does not look
+                             * like the next argument.
+                             */
+                            if((*associated_value == '\0') ||
+                                       (*associated_value == '-')){
+                                fprintf(GlobalState.logfile,
+                                        "Usage: -%c value\n",
+                                        argument_letter);
+                                exit(1);
+                            }
+                        }
+                        else {
+                            argnum++;
+                        }
+                        process_argument(argument[1], associated_value);
+                    }
+                    break;
+
+                /* Argument that require different treatment because they
+                 * are present on the command line rather than an argsfile.
+                 */
                 case TAGS_ARGUMENT:
                 case MOVES_ARGUMENT:
                 case POSITIONS_ARGUMENT:
                 case ENDINGS_ARGUMENT:
                     {   /* From the command line, we require an
-		         * associated file argument.
-			 * Check this here, as it is not the case
-			 * when reading arguments from an argument file.
-			 */
+                         * associated file argument.
+                         * Check this here, as it is not the case
+                         * when reading arguments from an argument file.
+                         */
                         const char *filename =  &(argument[2]);
-			const char argument_letter = argument[1];
+                        const char argument_letter = argument[1];
                         if(*filename == '\0'){
-			      /* Try to pick it up from the next argument. */
-			      if((argnum+1) < argc){
-				  ++argnum;
-				  filename = argv[argnum];
-			      }
-			    /* Make sure the filename does not look
-			     * like the next argument.
-			     */
-			    if((*filename == '\0') ||
-			       (*filename == '-')){
-				fprintf(GlobalState.logfile,
-					"Usage: -%cfilename or -%c filename\n",
-					argument_letter,argument_letter);
-				exit(1);
-			    }
-			}
-			else{
-			    process_argument(argument_letter,filename);
+                             /* Try to pick it up from the next argument. */
+                             argnum++;
+                             if(argnum < argc){
+                                 filename = argv[argnum];
+                                 argnum++;
+                             }
+                            /* Make sure the filename does not look
+                             * like the next argument.
+                             */
+                            if((*filename == '\0') || (*filename == '-')){
+                                fprintf(GlobalState.logfile,
+                                        "Usage: -%cfilename or -%c filename\n",
+                                        argument_letter,argument_letter);
+                                exit(1);
+                            }
                         }
+                        else{
+                            argnum++;
+                        }
+                        process_argument(argument_letter,filename);
                     }
                     break;
-                case USE_VIRTUAL_HASH_TABLE_ARGUMENT:
-		    process_argument(argument[1],"");
-                    break;
-                case GAMES_PER_FILE_ARGUMENT:
-		    process_argument(argument[1],&argument[2]);
-                    break;
                 default:
                     fprintf(GlobalState.logfile,
-			    "Unknown flag %s. Use -%c for usage details.\n",
-			    argument,HELP_ARGUMENT);
+                            "Unknown flag %s. Use -%c for usage details.\n",
+                            argument,HELP_ARGUMENT);
+                    exit(1);
                     break;
             }
         }
         else{
             /* Should be a file name containing games. */
             add_filename_to_source_list(argument,NORMALFILE);
+            argnum++;
         }
     }
     /* Prepare the hash tables for duplicate detection. */
