@@ -1,6 +1,6 @@
 /*
  *  Program: pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2005 David Barnes
+ *  Copyright (C) 1994-2007 David Barnes
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 1, or (at your option)
@@ -181,16 +181,16 @@ init_hashtab(void)
             HashCode code;
 
 #if 1
-	    /* Try to use a wider range of the available values
-	     * in an attempt to avoid spurious hash matches.
-	     */
-	    seed = (seed * 1103515245L) + 123456789L;
+            /* Try to use a wider range of the available values
+             * in an attempt to avoid spurious hash matches.
+             */
+            seed = (seed * 1103515245L) + 123456789L;
 #else
-	    /* This is the version used up to and including version 13.8. */
-	    seed = (seed * 1103515245L) + 12345L;
+            /* This is the version used up to and including version 13.8. */
+            seed = (seed * 1103515245L) + 12345L;
 #endif
-	    code = (seed >> SHIFT_LENGTH);
-	    HashTab[col-FIRSTCOL][rank-FIRSTRANK][piece-PAWN][colour-BLACK] = code;
+            code = (seed >> SHIFT_LENGTH);
+            HashTab[col-FIRSTCOL][rank-FIRSTRANK][piece-PAWN][colour-BLACK] = code;
           }
         }
       }
@@ -223,6 +223,8 @@ make_move(Col from_col, Rank from_rank, Col to_col, Rank to_rank,
     short to_c = ColConvert(to_col);
     short from_r = RankConvert(from_rank);
     short from_c = ColConvert(from_col);
+    /* Does this move involve a capture? */
+    Boolean capture = FALSE;
 
     /* If a KING or ROOK is moved, this might affect castling rights. */
     if(piece == KING){
@@ -270,8 +272,8 @@ make_move(Col from_col, Rank from_rank, Col to_col, Rank to_rank,
     }
     else{
         /* Castling not in question from the piece being moved,
-	 * but see below for checks on any captured piece.
-	 */
+         * but see below for checks on any captured piece.
+         */
     }
     /* Check for en-passant rights resulting from this move. */
     if(piece != PAWN){
@@ -319,6 +321,7 @@ make_move(Col from_col, Rank from_rank, Col to_col, Rank to_rank,
     board->board[from_r][from_c] = EMPTY;
     board->hash_value ^= HashLookup(from_col,from_rank,piece,colour);
     if(board->board[to_r][to_c] != EMPTY){
+
         /* Delete the captured piece from the hash value. */
         Piece coloured_piece = board->board[to_r][to_c];
         Piece removed_piece;
@@ -327,37 +330,45 @@ make_move(Col from_col, Rank from_rank, Col to_col, Rank to_rank,
         removed_piece = EXTRACT_PIECE(coloured_piece);
         removed_colour = EXTRACT_COLOUR(coloured_piece);
         board->hash_value ^= HashLookup(to_col,to_rank,removed_piece,removed_colour);
-	/* See whether the removed piece is a Rook, as this could
-	 * affect castling rights.
-	 */
-	if(removed_piece == ROOK){
-	    if(removed_colour == WHITE){
-		if(to_rank == FIRSTRANK){
-		    if(to_col == FIRSTCOL){
-			board->WQueenCastle = FALSE;
-		    }
-		    else if(to_col == LASTCOL){
-			board->WKingCastle = FALSE;
-		    }
-		    else{
-			/* No change. */
-		    }
-		}
-	    }
-	    else{
-		if(to_rank == LASTRANK){
-		    if(to_col == FIRSTCOL){
-			board->BQueenCastle = FALSE;
-		    }
-		    else if(to_col == LASTCOL){
-			board->BKingCastle = FALSE;
-		    }
-		    else{
-			/* No change. */
-		    }
-		}
-	    }
-	}
+        /* See whether the removed piece is a Rook, as this could
+         * affect castling rights.
+         */
+        if(removed_piece == ROOK){
+            if(removed_colour == WHITE){
+                if(to_rank == FIRSTRANK){
+                    if(to_col == FIRSTCOL){
+                        board->WQueenCastle = FALSE;
+                    }
+                    else if(to_col == LASTCOL){
+                        board->WKingCastle = FALSE;
+                    }
+                    else{
+                        /* No change. */
+                    }
+                }
+            }
+            else{
+                if(to_rank == LASTRANK){
+                    if(to_col == FIRSTCOL){
+                        board->BQueenCastle = FALSE;
+                    }
+                    else if(to_col == LASTCOL){
+                        board->BKingCastle = FALSE;
+                    }
+                    else{
+                        /* No change. */
+                    }
+                }
+            }
+        }
+        capture = TRUE;
+    }
+    /* Deal with the half-move clock. */
+    if(capture || piece == PAWN) {
+        board->halfmove_clock = 0;
+    }
+    else {
+        board->halfmove_clock++;
     }
     /* Place the piece at its destination. */
     board->board[to_r][to_c] = MAKE_COLOURED_PIECE(colour,piece);
@@ -401,7 +412,8 @@ find_pawn_moves(Col from_col, Rank from_rank, Col to_col,Rank to_rank,
                 }
             }
             else if(board->EnPassant &&
-                            (board->ep_rank == to_rank) && (board->ep_col == to_col)){
+                        (board->ep_rank == to_rank) &&
+                        (board->ep_col == to_col)){
                 /* Make sure that there is a valid pawn in position. */
                 if(from_col != 0){
                     from_r = to_r-offset;
@@ -664,6 +676,10 @@ king_is_in_check(const Board *board,Colour king_colour)
                                     opponent_colour,board)) != NULL)){
         /* King is in check from a pawn. to its left */
     }
+
+    if(replies != NULL) {
+        /* Already found a pawn move. */
+    }
     else if((replies =
         find_knight_moves(king_col,king_rank,
                                     opponent_colour,board)) != NULL){
@@ -1391,7 +1407,7 @@ determine_move_details(Colour colour,Move *move_details, Board *board)
             }
             else{
                 /* Pick up the promoted_piece from the move string. */
-                unsigned last_char = strlen((const char *)move)-1;
+                size_t last_char = strlen((const char *)move)-1;
 
                 /* Skip any check character. */
                 while(is_check(move[last_char])){
@@ -1444,7 +1460,7 @@ determine_move_details(Colour colour,Move *move_details, Board *board)
                     Move *alternative;
 
                     strcpy((char *) alternative_move,
-			   (const char *)move_details->move);
+                           (const char *)move_details->move);
                     *alternative_move = 'B';
                     alternative = decode_move((unsigned char *) alternative_move);
                     if(alternative != NULL){
@@ -1544,7 +1560,8 @@ determine_move_details(Colour colour,Move *move_details, Board *board)
      * castling is illegal, of course.
      */
 static MovePair *
-GenerateSingleMoves(Colour colour,Piece piece,Board *board,Col from_col, Rank from_rank)
+GenerateSingleMoves(Colour colour,Piece piece,
+                    const Board *board,Col from_col, Rank from_rank)
 {   short from_r = RankConvert(from_rank);
     short from_c = ColConvert(from_col);
     unsigned ix;
@@ -1589,7 +1606,7 @@ GenerateSingleMoves(Colour colour,Piece piece,Board *board,Col from_col, Rank fr
          */
 static MovePair *
 GenerateMultipleMoves(Colour colour,Piece piece,
-                Board *board,Col from_col, Rank from_rank)
+                const Board *board,Col from_col, Rank from_rank)
 {   short from_r = RankConvert(from_rank);
     short from_c = ColConvert(from_col);
     unsigned ix;
@@ -1636,7 +1653,7 @@ GenerateMultipleMoves(Colour colour,Piece piece,
 
         /* Generate a list of moves of a pawn from from_col,from_rank. */
 static MovePair *
-GeneratePawnMoves(Colour colour,Board *board,Col from_col, Rank from_rank)
+GeneratePawnMoves(Colour colour,const Board *board,Col from_col, Rank from_rank)
 {   MovePair *moves = NULL;
     Piece piece = PAWN;
     Colour target_colour = OPPOSITE_COLOUR(colour);
@@ -1649,13 +1666,13 @@ GeneratePawnMoves(Colour colour,Board *board,Col from_col, Rank from_rank)
     if(board->board[to_r][to_c] == EMPTY){
         /* Fill in the details, and add it to the list. */
         moves = append_move_pair(from_col,from_rank,ToCol(to_c),ToRank(to_r),moves);
-    }
-    if(((colour == WHITE) && (from_rank == FIRSTRANK+1)) ||
-           ((colour == BLACK) && (from_rank == LASTRANK-1))){
-       /* Try two steps. */
-       to_r = RankConvert(from_rank)+2*offset;
-        if(board->board[to_r][to_c] == EMPTY){
-            moves = append_move_pair(from_col,from_rank,ToCol(to_c),ToRank(to_r),moves);
+        if(((colour == WHITE) && (from_rank == FIRSTRANK+1)) ||
+               ((colour == BLACK) && (from_rank == LASTRANK-1))){
+           /* Try two steps. */
+           to_r = RankConvert(from_rank)+2*offset;
+            if(board->board[to_r][to_c] == EMPTY){
+                moves = append_move_pair(from_col,from_rank,ToCol(to_c),ToRank(to_r),moves);
+            }
         }
     }
     /* Try to left. */
@@ -1818,14 +1835,13 @@ approx_how_many_moves(Board *board)
    }
    return num_moves;
 }
+#endif
 
-        /* Find all moves for the colour board->to_move on board. */
-        /* Currently unused. */
-static MovePair *
-find_all_moves(Board *board)
+        /* Find all moves for colour board. */
+MovePair *
+find_all_moves(const Board *board, Colour colour)
 {  Rank rank;
    Col col;
-   Colour colour = board->to_move;
    /* All moves for colour. */
    MovePair *all_moves = NULL;
 
@@ -1903,5 +1919,4 @@ find_all_moves(Board *board)
    }
    return all_moves;
 }
-#endif
 
