@@ -1,6 +1,6 @@
 /*
  *  Program: pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2005 David Barnes
+ *  Copyright (C) 1994-2007 David Barnes
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 1, or (at your option)
@@ -27,3 +27,5 @@ void add_textual_variation_from_line(char *line);
 Boolean check_textual_variations(Game game_details);
 Boolean check_move_bounds(unsigned number_of_moves);
 void add_fen_positional_match(const char *fen_string);
+Boolean check_for_only_checkmate(const Move *moves);
+Boolean check_for_only_stalemate(const Board *board, const Move *moves);
