@@ -24,7 +24,7 @@
 #include "mymalloc.h"
 
 void *
-MallocOrDie(unsigned nbytes)
+MallocOrDie(size_t nbytes)
 {   void *result;
 
     result = malloc(nbytes);
@@ -36,7 +36,7 @@ MallocOrDie(unsigned nbytes)
 }
 
 void *
-ReallocOrDie(void *space,unsigned nbytes)
+ReallocOrDie(void *space,size_t nbytes)
 {   void *result;
 
     result = realloc(space,nbytes);
