@@ -20,5 +20,5 @@
  *
  */
 
-void *MallocOrDie(unsigned nbytes);
-void *ReallocOrDie(void *space,unsigned nbytes);
+void *MallocOrDie(size_t nbytes);
+void *ReallocOrDie(void *space,size_t nbytes);
