@@ -33,6 +33,7 @@
 #include "grammar.h"
 #include "apply.h"
 #include "output.h"
+#include "mymalloc.h"
 
 
             /* Functions for outputting games in the required format. */
@@ -48,7 +49,7 @@
 #define ARRAY_SIZE(arr) (sizeof(arr)/sizeof(*arr))
 
 /* How much text we have output on the current line. */
-static unsigned line_length = 0;
+static size_t line_length = 0;
 /* The buffer in which each output line of a game is built. */
 static char *output_line = NULL;
 
@@ -117,7 +118,7 @@ void
 set_output_line_length(unsigned max)
 {
     if(output_line != NULL) {
-	(void) free((void *) output_line);
+        (void) free((void *) output_line);
     }
     output_line = (char *) MallocOrDie(max + 1);
 }
@@ -151,15 +152,21 @@ which_output_format(const char *arg)
 
     for(i = 0; formats[i].arg != NULL; i++){
       const char *format_prefix = formats[i].arg;
-      const int format_prefix_len = strlen(format_prefix);
+      const size_t format_prefix_len = strlen(format_prefix);
       if(strncmp(arg,format_prefix,format_prefix_len) == 0){
             OutputFormat format = formats[i].format;
+            /* Sanity check. */
+            if(*format_prefix == '\0' && *arg != '\0') {
+                fprintf(GlobalState.logfile,
+                        "Unknown output format %s.\n", arg);
+                exit(1);
+            }
             /* If the format is SAN, it is possible to supply
              * a 6-piece suffix listing language-specific
              * letters to use in the output.
              */
             if((format == SAN || format == ELALG) &&
-	           (strlen(arg) > format_prefix_len)){
+                   (strlen(arg) > format_prefix_len)){
                 set_output_piece_characters(&arg[format_prefix_len]);
             }
             return format;
@@ -318,10 +325,10 @@ show_tags(FILE *outfp,char **Tags,int tags_length)
          * current line.
          */
 static void
-check_line_length(FILE *fp,unsigned len)
+check_line_length(FILE *fp,size_t len)
 {
     if((line_length + len) > GlobalState.max_line_length){
-	terminate_line(fp);
+        terminate_line(fp);
     }
 }
 
@@ -345,7 +352,7 @@ print_separator(FILE *fp)
      */
     check_line_length(fp,2);
     if(line_length != 0){
-	output_line[line_length] = ' ';
+        output_line[line_length] = ' ';
         line_length++;
     }
 }
@@ -356,12 +363,12 @@ terminate_line(FILE *fp)
 {
     /* Delete any trailing space(s). */
     while(line_length >= 1 && output_line[line_length - 1] == ' ') {
-	line_length--;
+        line_length--;
     }
     if(line_length > 0) {
-	output_line[line_length] = '\0';
-	fprintf(fp, "%s\n", output_line);
-	line_length = 0;
+        output_line[line_length] = '\0';
+        fprintf(fp, "%s\n", output_line);
+        line_length = 0;
     }
 }
 
@@ -370,7 +377,7 @@ terminate_line(FILE *fp)
          */
 void
 print_str(FILE *fp, const char *str)
-{   unsigned len = strlen(str);
+{   size_t len = strlen(str);
 
     check_line_length(fp,len);
     sprintf(&(output_line[line_length]),"%s",str);
@@ -414,12 +421,28 @@ print_move_list(FILE *outputfile, unsigned move_number, Boolean white_to_move,
                 const Move *move_details,const Board *final_board)
 {   unsigned print_move_number = 1;
     const Move *move = move_details;
+    Boolean keepPrinting;
+    unsigned plies;
 
-    while(move != NULL){
+    /* Work out the ply depth. */
+    plies = 2*(move_number) - 1;
+    if(!white_to_move) {
+        plies++;
+    }
+    if(GlobalState.output_ply_limit > 0 &&
+            plies > GlobalState.output_ply_limit) {
+        keepPrinting = FALSE;
+    }
+    else {
+        keepPrinting = TRUE;
+    }
+
+    while(move != NULL && keepPrinting){
         /* Reset print_move number if a variation was printed. */
         print_move_number = print_move(outputfile,move_number,
                                        print_move_number,
                                        white_to_move,move);
+
         /* See if there is a result attached.  This may be attached either
          * to a move or a comment.
          */
@@ -429,8 +452,10 @@ print_move_list(FILE *outputfile, unsigned move_number, Boolean white_to_move,
                 print_separator(outputfile);
                 print_str(outputfile,build_FEN_comment(final_board));
             }
-            print_separator(outputfile);
-            print_str(outputfile,move->terminating_result);
+            if(GlobalState.keep_results) {
+                print_separator(outputfile);
+                print_str(outputfile,move->terminating_result);
+            }
         }
         if(move->move[0] != '\0'){
             /* A genuine move was just printed, rather than a comment. */
@@ -441,6 +466,11 @@ print_move_list(FILE *outputfile, unsigned move_number, Boolean white_to_move,
                move_number++;
                white_to_move = TRUE;
             }
+            plies++;
+            if(GlobalState.output_ply_limit > 0 &&
+                    plies > GlobalState.output_ply_limit) {
+                keepPrinting = FALSE;
+            }
         }
         move = move->next;
         /* The following is slightly inaccurate.
@@ -450,10 +480,26 @@ print_move_list(FILE *outputfile, unsigned move_number, Boolean white_to_move,
          * Not sure how to cleanly fix it, because there might have
          * been nags attached to the comment that were printed, for instance!
          */
-        if(move != NULL){
+        if(move != NULL && keepPrinting){
             print_separator(outputfile);
         }
     }
+
+    if(move != NULL && !keepPrinting) {
+        /* We ended printing the game prematurely.
+         * Decide whether to print a result indicator.
+         */
+        if(GlobalState.keep_results) {
+            /* Find the final move to see if there was a result there. */
+            while(move->next != NULL) {
+                move = move->next;
+            }
+            if(move->terminating_result != NULL) {
+                print_separator(outputfile);
+                print_str(outputfile, "*");
+            }
+        }
+    }
 }
 
         /* Output the current move along with associated information.
@@ -483,7 +529,8 @@ print_move(FILE *outputfile, unsigned move_number, unsigned print_move_number,
             const unsigned char *move_text = move_details->move;
 
             if(*move_text != '\0'){
-                if(white_to_move || print_move_number){
+                if(GlobalState.keep_move_numbers &&
+                        (white_to_move || print_move_number)){
                     static char small_number[] = "99999...";
 
                     /* @@@ Should 1... be written as 1. ... ? */
@@ -544,13 +591,13 @@ print_move(FILE *outputfile, unsigned move_number, unsigned print_move_number,
                     case LALG:
                     case ELALG:
                         {  char algebraic[MAX_MOVE_LEN+1];
-                           int ind = 0;
+                           size_t ind = 0;
 
                            /* Prefix with a piece name if ELALG. */
                            if(GlobalState.output_format == ELALG &&
                                         move_details->class == PIECE_MOVE){
                                strcpy(algebraic,
-				      piece_str(move_details->piece_to_move));
+                                      piece_str(move_details->piece_to_move));
                                ind = strlen(algebraic);
                            }
                            /* Format the basics. */
@@ -680,22 +727,18 @@ promoted_piece_letter(Piece piece)
     }
 }
 
-    /* Build a FEN string from the given board.
-     * Currently, the number of half moves since the
-     * last pawn move or capture is not recorded.
-     */
+    /* Build a FEN string from the given board. */
 static void
 build_FEN_string(const Board *board,char *fen)
-{   int ix;
+{   size_t ix;
     build_basic_EPD_string(board,fen);
     /* Append the (pseudo) half move count and the full move count. */
     ix = strlen(fen);
     fen[ix] = ' '; ix++;
 
-    /* Half moves since the last capture or pawn move.
-     * @@@ Currently this is not set correctly.
-     */
-    fen[ix] = '0'; ix++;
+    /* Half moves since the last capture or pawn move. */
+    sprintf(&fen[ix], "%u", board->halfmove_clock);
+    ix = strlen(fen);
     fen[ix] = ' '; ix++;
 
     /* The move number. */
@@ -737,7 +780,7 @@ output_cm_comment(CommentList *comment,FILE *outputfile,unsigned indent)
         for( ; comment_str != NULL; comment_str = comment_str->next){
             chunk = strtok(comment_str->str," ");
             while(chunk != NULL){
-                unsigned len = strlen(chunk);
+                size_t len = strlen(chunk);
 
                 if((line_length+1+len) > GlobalState.max_line_length){
                     /* Start a new line. */
@@ -915,8 +958,8 @@ output_game(Game current_game,FILE *outputfile)
         case LALG:
         case HALG:
         case ELALG:
-	    print_algebraic_game(current_game,outputfile,move_number,white_to_move,
-				 final_board);
+            print_algebraic_game(current_game,outputfile,move_number,white_to_move,
+                                 final_board);
             break;
         case EPD:
             print_epd_game(current_game,outputfile,move_number,white_to_move,
@@ -975,9 +1018,9 @@ static const char *format_epd_game_comment(char **Tags)
 {   
     static char comment_prefix[] = "c0 ";
     static char player_separator[] = "-";
-    static int prefix_and_separator_len = sizeof(comment_prefix)+
-                                          sizeof(player_separator);
-    unsigned space_needed = prefix_and_separator_len;
+    static size_t prefix_and_separator_len =
+                sizeof(comment_prefix)+sizeof(player_separator);
+    size_t space_needed = prefix_and_separator_len;
     char *comment;
 
     if(Tags[WHITE_TAG] != NULL){
@@ -1031,46 +1074,54 @@ static const char *format_epd_game_comment(char **Tags)
 
 static void
 print_algebraic_game(Game current_game,FILE *outputfile,
-		     unsigned move_number,Boolean white_to_move,
-		     Board *final_board)
+                     unsigned move_number,Boolean white_to_move,
+                     Board *final_board)
 {
     /* Report details on the output. */
-    if(GlobalState.seven_tag_roster){
+    if(GlobalState.tag_output_format == ALL_TAGS) {
+      show_tags(outputfile,current_game.tags,current_game.tags_length);
+    }
+    else if(GlobalState.tag_output_format == SEVEN_TAG_ROSTER){
       output_STR(outputfile,current_game.tags);
       if(GlobalState.add_ECO && !GlobalState.parsing_ECO_file){
-		  /* If ECO classification has been requested, then assume
-		   * that ECO tags are also required.
-		   */
-		  output_tag(ECO_TAG,current_game.tags,outputfile);
-		  output_tag(OPENING_TAG,current_game.tags,outputfile);
-		  output_tag(VARIATION_TAG,current_game.tags,outputfile);
-		  output_tag(SUB_VARIATION_TAG,current_game.tags,outputfile);
+                  /* If ECO classification has been requested, then assume
+                   * that ECO tags are also required.
+                   */
+                  output_tag(ECO_TAG,current_game.tags,outputfile);
+                  output_tag(OPENING_TAG,current_game.tags,outputfile);
+                  output_tag(VARIATION_TAG,current_game.tags,outputfile);
+                  output_tag(SUB_VARIATION_TAG,current_game.tags,outputfile);
       }
 
       /* Output any FEN that there might be. */
       output_tag(FEN_TAG,current_game.tags,outputfile);
       putc('\n',outputfile);
     }
-    else{
-      show_tags(outputfile,current_game.tags,current_game.tags_length);
+    else if(GlobalState.tag_output_format == NO_TAGS) {
+    }
+    else {
+        fprintf(GlobalState.logfile,
+                "Unknown output form for tags: %d\n",
+                GlobalState.tag_output_format);
+        exit(1);
     }
     if((GlobalState.keep_comments) &&
-		    (current_game.prefix_comment != NULL)){
+                    (current_game.prefix_comment != NULL)){
       print_comment_list(outputfile,
-			 current_game.prefix_comment);
+                         current_game.prefix_comment);
       terminate_line(outputfile);
       putc('\n',outputfile);
     }
     print_move_list(outputfile,move_number,white_to_move,
-		    current_game.moves,final_board);
+                    current_game.moves,final_board);
     /* Take account of a possible zero move game. */
     if(current_game.moves == NULL) {
-	if(current_game.tags[RESULT_TAG] != NULL) {
-	    print_str(outputfile,current_game.tags[RESULT_TAG]);
-	}
+        if(current_game.tags[RESULT_TAG] != NULL) {
+            print_str(outputfile,current_game.tags[RESULT_TAG]);
+        }
         else{
             fprintf(GlobalState.logfile,
-		    "Internal error: Zero move game with no result\n");
+                    "Internal error: Zero move game with no result\n");
         }
     }
     terminate_line(outputfile);
@@ -1096,9 +1147,11 @@ print_epd_move_list(Game current_game,FILE *outputfile,
         putc('\n',outputfile);
         move = move->next;
     }
-    build_basic_EPD_string(final_board,epd);
-    fprintf(outputfile,"%s %s",epd,game_comment);
-    putc('\n',outputfile);
+    if(final_board != NULL) {
+        build_basic_EPD_string(final_board,epd);
+        fprintf(outputfile,"%s %s",epd,game_comment);
+        putc('\n',outputfile);
+    }
     (void) free((void *)game_comment);
 }
 
