@@ -20,13 +20,13 @@
  *
  */
 
-	/* Type definitions required by multiple files. */
+        /* Type definitions required by multiple files. */
 
     /* Define a type to represent different output formats.
      * Currently represented are:
      *     SOURCE: the original source notation.
-     *	   SAN: SAN.
-     *	   CM: Chess Master input format.
+     *           SAN: SAN.
+     *           CM: Chess Master input format.
      *     LALG: Long-algebraic, e.g. e2e4.
      *     HALG: Hyphenated long-algebraic, e.g. e2-e4.
      *     ELALG: Enhanced long-algebraic. Includes piece names, e.g. Ng1f3,
@@ -41,9 +41,9 @@ typedef enum { SOURCE, SAN, EPD, CM, LALG, HALG, ELALG } OutputFormat;
      */
 typedef enum { NOCHECK, CHECK, CHECKMATE } CheckStatus;
 
-	/* Permit lists of strings, e.g. lists of comments,
-	 * list of NAGs, etc.
-	 */
+        /* Permit lists of strings, e.g. lists of comments,
+         * list of NAGs, etc.
+         */
 typedef struct string_list {
     char *str;
     struct string_list *next;
@@ -69,9 +69,9 @@ typedef struct variation{
  */
 #define MAX_MOVE_LEN 15
 
-	/* Retain the text of a move and any associated 
-	 * NAGs and comments.
-	 */
+        /* Retain the text of a move and any associated 
+         * NAGs and comments.
+         */
 typedef struct move{
     /* @@@ This array is of type unsigned char,
      * in order to accommodate full 8-bit letters without
@@ -151,6 +151,15 @@ typedef enum {
     DONT_DIVIDE = 0, MIN_ECO_LEVEL = 1, MAX_ECO_LEVEL = 10
 } EcoDivision;
 
+/* Define a type to describe which tags are to be output.
+ * This used to be handled by the Boolean seven_tag_roster field
+ * in GlobalState but there are now different forms of output
+ * available.
+ */
+typedef enum {
+    ALL_TAGS = 0, SEVEN_TAG_ROSTER = 1, NO_TAGS = 2
+} TagOutputForm;
+
 /* This structure holds details of the program state.
  * Most of these fields are set from the program's arguments.
  */
@@ -175,8 +184,8 @@ typedef struct {
      * This is cleared with the -V flag.
      */
     Boolean keep_variations;
-    /* Whether to output just the Seven Tag Roster. */
-    Boolean seven_tag_roster;
+    /* Which tags are to be output. */
+    TagOutputForm tag_output_format;
     /* Whether to match permutations of textual variations or not. */
     Boolean match_permutations;
     /* Whether we are matching positional variations or not. */
@@ -203,6 +212,16 @@ typedef struct {
     Boolean use_virtual_hash_table;
     /* Whether to match on the number of moves in a game. */
     Boolean check_move_bounds;
+    /* Whether to match only games ending in checkmate. */
+    Boolean match_only_checkmate;
+    /* Whether to match only games ending in stalemate. */
+    Boolean match_only_stalemate;
+    /* Whether to output move numbers in the output. */
+    Boolean keep_move_numbers;
+    /* Whether to output results in the output. */
+    Boolean keep_results;
+    /* Whether to output an evaluation value after each move. */
+    Boolean output_evaluation;
     /* Maximum depth to which to search for positional variations.
      * This is picked up from the length of variations in the positional
      * variations file.
@@ -216,6 +235,8 @@ typedef struct {
     unsigned next_file_number;
     /* Lower and upper bounds for moves if check_move_bounds. */
     unsigned lower_move_bound, upper_move_bound;
+    /* Limit to the number of plies to appear in the output. */
+    unsigned output_ply_limit;
     /* Whether to output a FEN string at the end of the game. */
     Boolean output_FEN_string;
     /* Current input file name. */
